@@ -1,4 +1,23 @@
-#include <config.h>
+/* testfilechooser.c
+ * Copyright (C) 2003  Red Hat, Inc.
+ * Author: Owen Taylor
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+#include "config.h"
 
 #include <string.h>
 #include <sys/types.h>
@@ -140,7 +159,7 @@ format_size (gint64 size)
 #define _(s) (s)
 
 static void
-size_prepared_cb (GdkPixbufLoader *loader, 
+size_prepared_cb (GdkPixbufLoader *loader,
 		  int              width,
 		  int              height,
 		  int             *data)
@@ -163,7 +182,7 @@ size_prepared_cb (GdkPixbufLoader *loader,
 
 GdkPixbuf *
 my_new_from_file_at_size (const char *filename,
-			  int         width, 
+			  int         width,
 			  int         height,
 			  GError    **error)
 {
@@ -180,11 +199,13 @@ my_new_from_file_at_size (const char *filename,
         g_return_val_if_fail (width > 0 && height > 0, NULL);
 
 	if (stat (filename, &st) != 0) {
+                int errsv = errno;
+
 		g_set_error (error,
 			     G_FILE_ERROR,
-			     g_file_error_from_errno (errno),
+			     g_file_error_from_errno (errsv),
 			     _("Could not get information for file '%s': %s"),
-			     filename, g_strerror (errno));
+			     filename, g_strerror (errsv));
 		return NULL;
 	}
 
@@ -193,14 +214,16 @@ my_new_from_file_at_size (const char *filename,
 
 	f = fopen (filename, "rb");
 	if (!f) {
+                int errsv = errno;
+
                 g_set_error (error,
                              G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
+                             g_file_error_from_errno (errsv),
                              _("Failed to open file '%s': %s"),
-                             filename, g_strerror (errno));
+                             filename, g_strerror (errsv));
 		return NULL;
         }
-	
+
 	loader = gdk_pixbuf_loader_new ();
 #ifdef DONT_PRESERVE_ASPECT
 	gdk_pixbuf_loader_set_size (loader, width, height);
@@ -208,7 +231,7 @@ my_new_from_file_at_size (const char *filename,
 	info[0] = width;
 	info[1] = height;
 	g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), info);
-#endif	
+#endif
 
 	while (!feof (f)) {
 		length = fread (buffer, 1, sizeof (buffer), f);
@@ -258,7 +281,7 @@ update_preview_cb (GtkFileChooser *chooser)
 {
   gchar *filename = gtk_file_chooser_get_preview_filename (chooser);
   gboolean have_preview = FALSE;
-  
+
   if (filename)
     {
       GdkPixbuf *pixbuf;
@@ -281,10 +304,10 @@ update_preview_cb (GtkFileChooser *chooser)
 	      gchar *preview_text;
 	      gchar *size_str;
 	      gchar *modified_time;
-	      
+
 	      size_str = format_size (buf.st_size);
 	      modified_time = format_time (buf.st_mtime);
-	      
+
 	      preview_text = g_strdup_printf ("<i>Modified:</i>\t%s\n"
 					      "<i>Size:</i>\t%s\n",
 					      modified_time,
@@ -293,13 +316,13 @@ update_preview_cb (GtkFileChooser *chooser)
 	      g_free (modified_time);
 	      g_free (size_str);
 	      g_free (preview_text);
-	      
+
 	      gtk_widget_hide (preview_image);
 	      gtk_widget_show (preview_label);
 	      have_preview = TRUE;
 	    }
 	}
-      
+
       g_free (filename);
 
       if (error)
@@ -384,9 +407,9 @@ unmap_and_remap_cb (GtkButton *button,
 }
 
 static void
-kill_dependent (GtkWindow *win, GtkObject *dep)
+kill_dependent (GtkWindow *win, GtkWidget *dep)
 {
-  gtk_object_destroy (dep);
+  gtk_widget_destroy (dep);
   g_object_unref (dep);
 }
 
@@ -398,10 +421,59 @@ notify_multiple_cb (GtkWidget  *dialog,
   gboolean multiple;
 
   multiple = gtk_file_chooser_get_select_multiple (GTK_FILE_CHOOSER (dialog));
-  
+
   gtk_widget_set_sensitive (button, multiple);
 }
 
+static GtkFileChooserConfirmation
+confirm_overwrite_cb (GtkFileChooser *chooser,
+		      gpointer        data)
+{
+  GtkWidget *dialog;
+  GtkWidget *button;
+  int response;
+  GtkFileChooserConfirmation conf;
+
+  dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (chooser))),
+				   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				   GTK_MESSAGE_QUESTION,
+				   GTK_BUTTONS_NONE,
+				   "What do you want to do?");
+
+  button = gtk_button_new_with_label ("Use the stock confirmation dialog");
+  gtk_widget_show (button);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, 1);
+
+  button = gtk_button_new_with_label ("Type a new file name");
+  gtk_widget_show (button);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, 2);
+
+  button = gtk_button_new_with_label ("Accept the file name");
+  gtk_widget_show (button);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, 3);
+
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  switch (response)
+    {
+    case 1:
+      conf = GTK_FILE_CHOOSER_CONFIRMATION_CONFIRM;
+      break;
+
+    case 3:
+      conf = GTK_FILE_CHOOSER_CONFIRMATION_ACCEPT_FILENAME;
+      break;
+
+    default:
+      conf = GTK_FILE_CHOOSER_CONFIRMATION_SELECT_AGAIN;
+      break;
+    }
+
+  gtk_widget_destroy (dialog);
+
+  return conf;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -409,41 +481,61 @@ main (int argc, char **argv)
   GtkWidget *vbbox;
   GtkWidget *button;
   GtkWidget *dialog;
-  GtkWidget *prop_editor;
   GtkWidget *extra;
   GtkFileFilter *filter;
   GtkWidget *preview_vbox;
-  int i;
+  gboolean force_rtl = FALSE;
   gboolean multiple = FALSE;
-  
-  gtk_init (&argc, &argv);
+  char *action_arg = NULL;
+  char *initial_filename = NULL;
+  char *initial_folder = NULL;
+  GError *error = NULL;
+  GOptionEntry options[] = {
+    { "action", 'a', 0, G_OPTION_ARG_STRING, &action_arg, "Filechooser action", "ACTION" },
+    { "multiple", 'm', 0, G_OPTION_ARG_NONE, &multiple, "Select-multiple", NULL },
+    { "right-to-left", 'r', 0, G_OPTION_ARG_NONE, &force_rtl, "Force right-to-left layout.", NULL },
+    { "initial-filename", 'f', 0, G_OPTION_ARG_FILENAME, &initial_filename, "Initial filename to select", "FILENAME" },
+    { "initial-folder", 'F', 0, G_OPTION_ARG_FILENAME, &initial_folder, "Initial folder to show", "FILENAME" },
+    { NULL }
+  };
 
-  /* to test rtl layout, set RTL=1 in the environment */
-  if (g_getenv ("RTL"))
+  if (!gtk_init_with_args (&argc, &argv, "", options, NULL, &error))
+    {
+      g_print ("Failed to parse args: %s\n", error->message);
+      g_error_free (error);
+      return 1;
+    }
+
+  if (initial_filename && initial_folder)
+    {
+      g_print ("Only one of --initial-filename and --initial-folder may be specified");
+      return 1;
+    }
+
+  if (force_rtl)
     gtk_widget_set_default_direction (GTK_TEXT_DIR_RTL);
-  
+
   action = GTK_FILE_CHOOSER_ACTION_OPEN;
 
-  /* lame-o arg parsing */
-  for (i = 1; i < argc; i++)
+  if (action_arg != NULL)
     {
-      if (! strcmp ("--action=open", argv[i]))
+      if (! strcmp ("open", action_arg))
 	action = GTK_FILE_CHOOSER_ACTION_OPEN;
-      else if (! strcmp ("--action=save", argv[i]))
+      else if (! strcmp ("save", action_arg))
 	action = GTK_FILE_CHOOSER_ACTION_SAVE;
-      else if (! strcmp ("--action=select_folder", argv[i]))
+      else if (! strcmp ("select_folder", action_arg))
 	action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
-      else if (! strcmp ("--action=create_folder", argv[i]))
+      else if (! strcmp ("create_folder", action_arg))
 	action = GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER;
-      else if (! strcmp ("--multiple", argv[i]))
-	multiple = TRUE;
+
+      g_free (action_arg);
     }
 
   dialog = g_object_new (GTK_TYPE_FILE_CHOOSER_DIALOG,
 			 "action", action,
-			 "file-system-backend", "gtk+",
 			 "select-multiple", multiple,
 			 NULL);
+
   switch (action)
     {
     case GTK_FILE_CHOOSER_ACTION_OPEN:
@@ -464,20 +556,22 @@ main (int argc, char **argv)
       break;
     }
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-  
+
   g_signal_connect (dialog, "selection-changed",
 		    G_CALLBACK (print_selected), NULL);
   g_signal_connect (dialog, "current-folder-changed",
 		    G_CALLBACK (print_current_folder), NULL);
   g_signal_connect (dialog, "response",
 		    G_CALLBACK (response_cb), NULL);
+  g_signal_connect (dialog, "confirm-overwrite",
+		    G_CALLBACK (confirm_overwrite_cb), NULL);
 
   /* Filters */
   filter = gtk_file_filter_new ();
   gtk_file_filter_set_name (filter, "All Files");
   gtk_file_filter_add_pattern (filter, "*");
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
-  
+
   filter = gtk_file_filter_new ();
   gtk_file_filter_set_name (filter, "No backup files");
   gtk_file_filter_add_custom (filter, GTK_FILE_FILTER_DISPLAY_NAME,
@@ -485,12 +579,12 @@ main (int argc, char **argv)
   gtk_file_filter_add_mime_type (filter, "image/png");
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
 
-  g_signal_connect (dialog, "notify::filter", 
+  g_signal_connect (dialog, "notify::filter",
 		    G_CALLBACK (filter_changed), NULL);
 
   /* Make this filter the default */
   gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
-  
+
   filter = gtk_file_filter_new ();
   gtk_file_filter_set_name (filter, "PNG and JPEG");
   gtk_file_filter_add_mime_type (filter, "image/jpeg");
@@ -501,21 +595,21 @@ main (int argc, char **argv)
   gtk_file_filter_set_name (filter, "Images");
   gtk_file_filter_add_pixbuf_formats (filter);
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
-  
+
   /* Preview widget */
   /* THIS IS A TERRIBLE PREVIEW WIDGET, AND SHOULD NOT BE COPIED AT ALL.
    */
-  preview_vbox = gtk_vbox_new (0, FALSE);
+  preview_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   /*gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog), preview_vbox);*/
-  
+
   preview_label = gtk_label_new (NULL);
   gtk_box_pack_start (GTK_BOX (preview_vbox), preview_label, TRUE, TRUE, 0);
-  gtk_misc_set_padding (GTK_MISC (preview_label), 6, 6);
-  
+  g_object_set (preview_label, "margin", 6, NULL);
+
   preview_image = gtk_image_new ();
   gtk_box_pack_start (GTK_BOX (preview_vbox), preview_image, TRUE, TRUE, 0);
-  gtk_misc_set_padding (GTK_MISC (preview_image), 6, 6);
-  
+  g_object_set (preview_image, "margin", 6, NULL);
+
   update_preview_cb (GTK_FILE_CHOOSER (dialog));
   g_signal_connect (dialog, "update-preview",
 		    G_CALLBACK (update_preview_cb), NULL);
@@ -532,16 +626,24 @@ main (int argc, char **argv)
 					    "file:///usr/share/pixmaps",
 					    NULL);
 
+  /* Initial filename or folder */
+
+  if (initial_filename)
+    set_filename (GTK_FILE_CHOOSER (dialog), initial_filename);
+
+  if (initial_folder)
+    set_current_folder (GTK_FILE_CHOOSER (dialog), initial_folder);
+
   /* show_all() to reveal bugs in composite widget handling */
   gtk_widget_show_all (dialog);
 
   /* Extra controls for manipulating the test environment
    */
-  prop_editor = create_prop_editor (G_OBJECT (dialog), GTK_TYPE_FILE_CHOOSER);
+  create_prop_editor (G_OBJECT (dialog), GTK_TYPE_FILE_CHOOSER);
 
   control_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  
-  vbbox = gtk_vbutton_box_new ();
+
+  vbbox = gtk_button_box_new (GTK_ORIENTATION_VERTICAL);
   gtk_container_add (GTK_CONTAINER (control_window), vbbox);
 
   button = gtk_button_new_with_mnemonic ("_Select all");
@@ -549,9 +651,9 @@ main (int argc, char **argv)
   gtk_container_add (GTK_CONTAINER (vbbox), button);
   g_signal_connect_swapped (button, "clicked",
 			    G_CALLBACK (gtk_file_chooser_select_all), dialog);
-  g_signal_connect (dialog, "notify::select-multiple", 
+  g_signal_connect (dialog, "notify::select-multiple",
 		    G_CALLBACK (notify_multiple_cb), button);
-  
+
   button = gtk_button_new_with_mnemonic ("_Unselect all");
   gtk_container_add (GTK_CONTAINER (vbbox), button);
   g_signal_connect_swapped (button, "clicked",
